﻿Imports System.Drawing

Public Class Form1

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        EventsAreEnabled = False
        Me.Text = AppTitleAndVersion()
        StartProcess_IfNotRunning_AndWait(Application.StartupPath + "\HAL\Theremino_HAL.exe", False)
        TdsScale = 0
        Load_INI()
        Set_Props()
        SetScale()
        UpdateTdsScale()
        EventsAreEnabled = True
        GroupBox3.Focus()
        Refresh()
        Opacity = 1
        Timer1.Interval = 30
        Timer1.Start()
        ' ---------------------------------
        TestTemperatureCorrections()
    End Sub

    Private Sub Form_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles Me.FormClosing
        Save_INI()
        If Not Debugger.IsAttached Then
            CloseAllOpenedProcesses()
        End If
    End Sub

    Private Sub Form_LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.LocationChanged
        If Not EventsAreEnabled Then Exit Sub
        LimitFormPosition(Me)
    End Sub

    ' ================================================================================================
    '   BUTTONS
    ' ================================================================================================
    Private Sub lbl_SlotViewer_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lbl_SlotViewer.Click
        StartProcess_IfNotRunning_AndWait(Application.StartupPath + "\HAL\Theremino_SlotViewer.exe", False)
    End Sub
    Private Sub lbl_CurveFitting_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lbl_CurveFitting.Click
        StartProcess_IfNotRunning_AndWait(Application.StartupPath + "\CurveFitting\Theremino_CurveFitting.exe", False)
    End Sub
    Private Sub lbl_EditCoeffs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lbl_EditCoeffs.Click
        Me.Visible = False
        NotepadProcess_StartAndWait(Application.StartupPath + "\Theremino_WaterQuality_Configuration.txt")
        LoadConfiguration()
        Me.Visible = True
    End Sub
    Private Sub lbl_HELP_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lbl_HELP.Click
        Process.Start(Application.StartupPath + "\helps")
    End Sub

    ' ================================================================================================
    '   SET PROPERTIES
    ' ================================================================================================
    Private Sub Props_Changed(ByVal sender As Object, ByVal e As System.EventArgs) Handles txt_RawInSlot.TextChanged, _
                                                                                           txt_TempInSlot.TextChanged, _
                                                                                           txt_TdsOutSlot.TextChanged
        If Not EventsAreEnabled Then Exit Sub
        Set_Props()
        Save_INI()
    End Sub

    Private Sub Set_Props()
        Slot_RawIn = txt_RawInSlot.NumericValueInteger
        Slot_TempIn = txt_TempInSlot.NumericValueInteger
        Slot_TdsOut = txt_TdsOutSlot.NumericValueInteger
    End Sub

    ' ================================================================================================
    '   SCALE
    ' ================================================================================================
    Private Sub lbl_Scale_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lbl_Scale.Click
        TdsScale = (TdsScale + 1) Mod 4
        SetScale()
        Save_INI()
    End Sub
    Private Sub SetScale()
        If TdsScale < 0 Then TdsScale = 0
        If TdsScale > 3 Then TdsScale = 3
        Select Case TdsScale
            Case 0
                lbl_Scale.Text = "Scale 500"
                TdsToPixel = 1
                Me.Width = 849
                ShowTestValues(False)
            Case 1
                lbl_Scale.Text = "Scale 1000"
                TdsToPixel = 0.5
                Me.Width = 751
                ShowTestValues(False)
            Case 2
                lbl_Scale.Text = "Scale 50k"
                TdsToPixel = 0.01
                Me.Width = 751
                ShowTestValues(False)
            Case 3
                lbl_Scale.Text = "Test values"
                TdsToPixel = 1
                Me.Width = 849
                ShowTestValues(True)
        End Select
        LoadConfiguration()
    End Sub

    Private Sub ShowTestValues(ByVal show As Boolean)
        lbl_Test1.Visible = show
        lbl_Test2.Visible = show
        lbl_Test3.Visible = show
        txt_VoltToConductance.Visible = show
        txt_ConductanceToTDS.Visible = show
        txt_TestSolutionConductance.Visible = show
    End Sub


    ' ================================================================================================
    '   TIMER
    ' ================================================================================================
    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        If Not EventsAreEnabled Then Return
        If Slot_RawIn >= 0 Then
            ComputeTDS()
            ShowData()
            UpdateTdsScale()
            If Slot_TdsOut >= 0 Then
                Slots.WriteSlot(Slot_TdsOut, TDS)
            End If
        End If
    End Sub

    ' ================================================================================================
    '   Compute and Show TDS
    ' ================================================================================================
    Friend TdsScale As Int32
    Private Slot_RawIn As Int32 = -1
    Private Slot_TempIn As Int32 = -1
    Private Slot_TdsOut As Int32 = -1
    Private TdsToPixel As Single
    Private RawValue As Single
    Private Temperature As Double
    Private Volt As Single
    Private TDS As Double

    Private Sub ComputeTDS()
        RawValue = Slots.ReadSlot_NoNan_0_1000(Slot_RawIn)
        Volt = RawValue * 0.0033F
        ' -------------------------------------------------------------- TDS
        If TdsScale < 3 Then
            ' -------------------------------------------------------------- polinomial from configuration
            TDS = ScaleConfigArray(TdsScale).c0
            TDS += ScaleConfigArray(TdsScale).c1 * Volt
            TDS += ScaleConfigArray(TdsScale).c2 * Volt ^ 2
            TDS += ScaleConfigArray(TdsScale).c3 * Volt ^ 3
            TDS *= ScaleConfigArray(TdsScale).ProbeK
            ' -------------------------------------------------------------- temperature
            If Slot_TempIn >= 0 Then
                Temperature = Slots.ReadSlot_NoNan_0_1000(Slot_TempIn)
                Temperature = Interpolate(TempSensorRaw, TempSensorDegree, Temperature)
                ' ---------------------------------------------------------- TDS correction by temperature
                Dim Correction As Double
                Correction = Interpolate(TempCorrectionsDegree, TempCorrectionsMultiplier, Temperature)
                TDS *= Correction
            End If
        Else
            TDS = 0
        End If
    End Sub

    Private Sub TestTemperatureCorrections()
        'Debug.Print(vbCrLf)
        'Dim c1, c2 As Double
        'For i As Int32 = 5 To 30 Step 5
        '    c1 = Math.E ^ ((25 - i) * 1.98 * 0.01)
        '    c2 = Interpolate(TempCorrectionsDegree, TempCorrectionsMultiplier, i)

        '    Debug.Print(i.ToString + " " + c1.ToString("0.000", GCI) + " " + c2.ToString("0.000", GCI))
        'Next
    End Sub


    Private Sub ShowData()
        If RawValue > 700 Then
            GroupBox2.BackColor = Color.FromArgb(255, 140, 50)
        Else
            GroupBox2.BackColor = GroupBox1.BackColor
        End If
        If Slot_TempIn >= 0 Then
            lbl_Temperature.Text = Temperature.ToString("0.0", GCI)
        Else
            lbl_Temperature.Text = "  -"
        End If
        lbl_RawInput.Text = RawValue.ToString("0.0", GCI)
        lbl_Voltage.Text = Volt.ToString("0.000", GCI)
        If TdsScale < 3 Then
            lbl_TDS.Text = TDS.ToString("0")
        Else
            lbl_TDS.Text = "  -"
        End If
    End Sub

    ' ================================================================================================
    '   TDS scale
    ' ================================================================================================
    Private GFX As Graphics
    Private Pen1 As Pen = New Pen(Color.Black, 3)
    Private Brush1 As SolidBrush = New SolidBrush(Color.Yellow)
    Private Brush2 As SolidBrush = New SolidBrush(Color.FromArgb(0, 50, 80))
    Private Font1 As Font = New Font("Courier new", 18, FontStyle.Bold)

    Private Sub UpdateTdsScale()
        Select Case TdsScale
            Case 0 : pbox_SpectrumBars.Image = My.Resources.TDS_500
            Case 1 : pbox_SpectrumBars.Image = My.Resources.TDS_1000
            Case 2 : pbox_SpectrumBars.Image = My.Resources.TDS_50000
            Case 3 : pbox_SpectrumBars.Image = My.Resources.Test
        End Select
        Dim x, y As Int32
        Dim GFX As Graphics = Graphics.FromImage(pbox_SpectrumBars.Image)
        Select Case TdsScale
            Case 0, 1, 2
                x = CInt(30 + TDS * TdsToPixel)
                GFX.FillEllipse(Brush1, x - 9, 82, 18, 18)
                GFX.DrawEllipse(Pen1, x - 11, 80, 22, 22)
            Case 3
                GFX.Clear(Color.FromArgb(245, 245, 240))
                '
                Dim CondMicroS As Double = Volt * txt_VoltToConductance.NumericValue
                If CondMicroS < 0.1 Then CondMicroS = 0
                Dim ResOhm As Double = 1000000 / CondMicroS
                Dim TdsPPM As Double = txt_TestSolutionConductance.NumericValue * txt_ConductanceToTDS.NumericValue
                Dim ProbeK As Double = txt_TestSolutionConductance.NumericValue / CondMicroS
                '
                x = 340
                y = 32
                GFX.DrawString("Conductance  (uS) = " + CondMicroS.ToString("0.0", GCI), Font1, Brush2, x, y)
                GFX.DrawString("Resistance  (ohm) = " + ResOhm.ToString("0", GCI), Font1, Brush2, x, y + 30)
                GFX.DrawString("Test solution TDS = " + TdsPPM.ToString("0", GCI), Font1, Brush2, x, y + 60)
                GFX.DrawString("Probe constant K  = " + ProbeK.ToString("0.00", GCI), Font1, Brush2, x, y + 90)
        End Select
        pbox_SpectrumBars.Refresh()
    End Sub


    ' ======================================
    '       PPM MAX = 500 or 1000 
    ' ======================================
    '   RX    HAL      mV    ERR       PPM
    ' --------------------------------------
    '  400    727    2400    >10%
    '  450    727    2400    5.8%      901
    '  500    692    2285              857
    '  600    577    1905              715
    '  800    433    1428              536
    '   1K    346    1143              429
    '  10k     34     113    0.3%       43
    ' 100k      3      11    3.3%        4
    ' 1meg      0       0    >10%


    ' ======================================
    '           PPM MAX = 50000
    ' ======================================
    '   RX    HAL      mV    ERR       PPM
    ' --------------------------------------
    '    8    727    2400    >10%
    '    9    727    2400    5.8%    45050
    '   10    692    2285            42850
    '   12    577    1905            35750
    '   16    433    1428            26800
    '   20    346    1143            21450
    '  200     34     113    0.3%     2150
    '   2k      3      11    3.3%      200
    '  20k      0       0    >10%


End Class