﻿
Imports System.Math

Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False


    Friend GCI As Globalization.CultureInfo = Globalization.CultureInfo.InvariantCulture

    ' =======================================================================================================
    '   APP TITLE AND VERSION (ALL FIELDS)
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal RemoveThereminoPrefix As Boolean = False) As String
        Dim appname As String = IO.Path.GetFileNameWithoutExtension(Application.ExecutablePath)
        Dim Title As String = Replace(appname, "_", " ")
        If RemoveThereminoPrefix Then Title = Replace(Title, "Theremino ", "")
        Return Title + " - V" + GetVersionString()
    End Function

    ' =======================================================================================================
    '   GET VERSION (ALSO TRIPLE OR QUADRUPLE FIELDS)
    ' =======================================================================================================
    Friend Function GetVersionString() As String
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        GetVersionString = s(0) + "." + s(1)
        If s(2) <> "0" Then
            GetVersionString += "." + s(2)
            If s(3) <> "0" Then
                GetVersionString += "." + s(3)
            End If
        End If
    End Function


    ' =======================================================================================
    '  FORM FUNCTIONS
    ' =======================================================================================
    Friend Sub LimitFormPosition_CompletelyVisible(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormCompletelyVisible(f)
    End Sub

    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
    End Sub

    Private SB As Rectangle = New Rectangle(Integer.MaxValue, Integer.MaxValue, Integer.MinValue, Integer.MinValue)

    Private Sub GetMaxScreenBounds()
        For Each s As Screen In System.Windows.Forms.Screen.AllScreens
            SB = Rectangle.Union(SB, s.WorkingArea)
        Next
    End Sub

    Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)         ' not more than a maximized window
            .Height = Math.Min(.Height, SB.Height)      ' not more than a maximized window
            .Width = Math.Max(.Width, 32)               ' at least 32x24
            .Height = Math.Max(.Height, 24)             ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - .Width)  ' not beyond the right border
            .Top = Math.Min(.Top, SB.Bottom - .Height)  ' not beyond the bottom border
            .Left = Math.Max(.Left, SB.Left)            ' at least at the left border
            .Top = Math.Max(.Top, SB.Top)               ' at least at the top border
        End With
    End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)             ' not more than VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, SB.Height)          ' not more than VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)                   ' at least 32x24
            .Height = Math.Max(.Height, 24)                 ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - 200)         ' not beyond right border - 200 pixels
            .Top = Math.Min(.Top, SB.Bottom - 150)          ' not beyond bottom border - 150 pixels
            .Left = Math.Max(.Left, SB.Left + 200 - .Width) ' at least at left border + 200 pixels
            .Top = Math.Max(.Top, SB.Top)                   ' at least at top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================
    Private Function TabString(ByVal Name As String, _
                               Optional ByVal Value As Double = Double.NaN, _
                               Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt, GCI)
        End If
    End Function
    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function
    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function

    Private Function Val_Single(ByVal l As String) As Single
        Return CSng(Val(l.Replace(",", ".")))
    End Function
    Private Function Val_Double(ByVal l As String) As Double
        Return Val(Replace(l, ",", "."))
    End Function
    Private Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function

    Private Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function
    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function


    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================
    Friend Sub Save_INI()
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
        Dim f As System.IO.StreamWriter = Nothing
        Try
            f = IO.File.CreateText(iniFileName)
            '
            f.WriteLine(" Program Params")
            f.WriteLine("===========================================")
            ' ------------------------------------------------------------------------------ FORM BOUNDS
            Dim r As Rectangle
            r = GetFormRectangle(Form1)
            f.WriteLine(TabString("Form1_Top", r.Top))
            f.WriteLine(TabString("Form1_Left", r.Left))
            'f.WriteLine(TabString("Form1_Width", r.Width))
            'f.WriteLine(TabString("Form1_Height", r.Height))
            f.WriteLine(TabString("Form1_WindowState", Form1.WindowState))
            '
            f.WriteLine("")
            f.WriteLine(" Slots")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("RawInSlot", Form1.txt_RawInSlot.Text))
            f.WriteLine(TabString("TempInSlot", Form1.txt_TempInSlot.Text))
            f.WriteLine(TabString("TdsOutSlot", Form1.txt_TdsOutSlot.Text))
            f.WriteLine(TabString("Scale", Form1.TdsScale))
            '
            f.WriteLine("")
            f.WriteLine(" Test values")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("VoltToConductance", Form1.txt_VoltToConductance.Text))
            f.WriteLine(TabString("TestSolutionConductance", Form1.txt_TestSolutionConductance.Text))
            f.WriteLine(TabString("ConductanceToTDS", Form1.txt_ConductanceToTDS.Text))
        Catch
        End Try
        Try
            f.Close()
        Catch
        End Try
    End Sub


    Friend Sub Load_INI()
        ' -------------------------------------------------------------
        ' ------------------------------------------------------------------------------- 
        ' With "Resume Next" subsequent parameters are loaded and f.Close() is executed
        ' -------------------------------------------------------------------------------
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' -------------------------------------------------------------------------------
        Dim l As String
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
        If My.Computer.FileSystem.FileExists(iniFileName) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)

            Do While Not f.EndOfStream
                l = f.ReadLine()
                Select Case ExtractParamName(l)
                    Case "Form1_Top" : Form1.Top = Val_Int(l)
                    Case "Form1_Left" : Form1.Left = Val_Int(l)
                        'Case "Form1_Width" : Form1.Width = Val_Int(l))
                        'Case "Form1_Height" : Form1.Height = Val_Int(l))
                    Case "Form1_WindowState" : Form1.WindowState = CType(l, FormWindowState)
                        '
                    Case "RawInSlot" : Form1.txt_RawInSlot.Text = l
                    Case "TempInSlot" : Form1.txt_TempInSlot.Text = l
                    Case "TdsOutSlot" : Form1.txt_TdsOutSlot.Text = l
                    Case "Scale" : Form1.TdsScale = Val_Int(l)
                        '
                    Case "VoltToConductance" : Form1.txt_VoltToConductance.Text = l
                    Case "TestSolutionConductance" : Form1.txt_TestSolutionConductance.Text = l
                    Case "ConductanceToTDS" : Form1.txt_ConductanceToTDS.Text = l
                End Select
            Loop
            f.Close()
        End If
        LimitFormPosition(Form1)
    End Sub



    ' =========================================================================================================
    '  LOAD CONFIGURATION
    ' =========================================================================================================
    Friend Structure ScaleConfigEntry
        Dim ProbeK As Double
        Dim c0 As Double
        Dim c1 As Double
        Dim c2 As Double
        Dim c3 As Double
    End Structure
    Friend ScaleConfigArray(2) As ScaleConfigEntry

    Friend TempSensorRaw(-1) As Double
    Friend TempSensorDegree(-1) As Double
    Friend TempCorrectionsDegree(-1) As Double
    Friend TempCorrectionsMultiplier(-1) As Double

    Private Sub InitScaleDefaults()
        ' ----------------------------------- scale 500
        ScaleConfigArray(0).ProbeK = 0.5
        ScaleConfigArray(0).c0 = 0
        ScaleConfigArray(0).c1 = 857.39
        ScaleConfigArray(0).c2 = 255.86
        ScaleConfigArray(0).c3 = 133.42
        ' ----------------------------------- scale 1000
        ScaleConfigArray(1).ProbeK = 0.5
        ScaleConfigArray(1).c0 = 0
        ScaleConfigArray(1).c1 = 857.39
        ScaleConfigArray(1).c2 = 255.86
        ScaleConfigArray(1).c3 = 133.42
        ' ----------------------------------- scale 50000
        ScaleConfigArray(2).ProbeK = 25
        ScaleConfigArray(2).c0 = 0
        ScaleConfigArray(2).c1 = 1
        ScaleConfigArray(2).c2 = 0
        ScaleConfigArray(2).c3 = 0
    End Sub

    Private Sub ResetTemperatureArrays()
        ReDim TempSensorRaw(-1)
        ReDim TempSensorDegree(-1)
        ReDim TempCorrectionsDegree(-1)
        ReDim TempCorrectionsMultiplier(-1)
    End Sub

    Private Sub InitTemperatureDefaults()
        ' ----------------------------------- 
        ReDim TempSensorRaw(1)
        ReDim TempSensorDegree(1)
        TempSensorRaw(0) = 0
        TempSensorDegree(0) = 0
        TempSensorRaw(1) = 100
        TempSensorDegree(1) = 33
        ' ----------------------------------- 
        ReDim TempCorrectionsDegree(5)
        ReDim TempCorrectionsMultiplier(5)
        TempCorrectionsDegree(0) = 5
        TempCorrectionsMultiplier(0) = 1.613
        TempCorrectionsDegree(1) = 10
        TempCorrectionsMultiplier(1) = 1.411
        TempCorrectionsDegree(2) = 15
        TempCorrectionsMultiplier(2) = 1.247
        TempCorrectionsDegree(3) = 20
        TempCorrectionsMultiplier(3) = 1.112
        TempCorrectionsDegree(4) = 25
        TempCorrectionsMultiplier(4) = 1.0
        TempCorrectionsDegree(5) = 30
        TempCorrectionsMultiplier(5) = 0.907
    End Sub

    Friend Sub LoadConfiguration()
        On Error Resume Next
        InitScaleDefaults()
        ResetTemperatureArrays()
        Dim ConfigArrayIndex As Int32 = -1
        Dim sa As String()
        Dim fname As String = Application.StartupPath + "\Theremino_WaterQuality_Configuration.txt"
        If IO.File.Exists(fname) Then
            sa = IO.File.ReadAllLines(fname)
            Dim s1, s2 As String
            For i As Int32 = 0 To sa.Length - 1
                s1 = sa(i).Trim
                RemoveComments(s1)
                s1 = ReplaceMultipleSpacesAndTabAndTrim(s1)
                s2 = s1.ToUpper
                If s2.Contains("END") Or s2.Contains("CONFIGURATION") Then Exit For
                If s2 = "" Or s1.StartsWith("=") Or s1.StartsWith("-") Then Continue For
                If s2(0) < "A" Or s1(0) > "Z" Then Continue For
                Dim sa2 As String() = s2.Split(" "c)
                If sa2.Length >= 2 Then
                    Select Case sa2(0)
                        ' ------------------------------------------------------------------------------- SCALE
                        Case "SCALE"
                            ConfigArrayIndex += 1
                            If ConfigArrayIndex > 2 Then
                                ConfigArrayIndex = 2
                                ScaleConfigLineUnrecognized(s1)
                                InitScaleDefaults()
                                Exit For
                            End If
                            Dim n1 As Int32
                            Select Case ConfigArrayIndex
                                Case 0 : n1 = 500
                                Case 1 : n1 = 1000
                                Case 2 : n1 = 50000
                            End Select
                            Dim n2 As Int32
                            n2 = CInt(Val(sa2(1)))
                            If sa2(1).ToUpper.EndsWith("K") Then n2 *= 1000
                            If n1 <> n2 Then
                                ScaleConfigLineUnrecognized(s1)
                                InitScaleDefaults()
                                Exit For
                            End If
                            ' --------------------------------------------------------------------------- VALUES
                        Case "PROBEK" : ScaleConfigArray(ConfigArrayIndex).ProbeK = Val_Double(sa2(1))
                        Case "C0" : ScaleConfigArray(ConfigArrayIndex).c0 = Val_Double(sa2(1))
                        Case "C1" : ScaleConfigArray(ConfigArrayIndex).c1 = Val_Double(sa2(1))
                        Case "C2" : ScaleConfigArray(ConfigArrayIndex).c2 = Val_Double(sa2(1))
                        Case "C3" : ScaleConfigArray(ConfigArrayIndex).c3 = Val_Double(sa2(1))
                            ' --------------------------------------------------------------------------- Temperature corrections
                        Case "TEMPERATURE" ' Nothing to do
                        Case "TEMPSENSORTRIMMING"
                            If sa2.Length >= 3 Then
                                Dim n As Int32 = TempSensorRaw.Length
                                ReDim Preserve TempSensorRaw(n)
                                ReDim Preserve TempSensorDegree(n)
                                TempSensorRaw(n) = Val_Double(sa2(1))
                                TempSensorDegree(n) = Val_Double(sa2(2))
                            End If
                        Case "TEMPCORRECTION"
                            If sa2.Length >= 3 Then
                                Dim n As Int32 = TempCorrectionsDegree.Length
                                ReDim Preserve TempCorrectionsDegree(n)
                                ReDim Preserve TempCorrectionsMultiplier(n)
                                TempCorrectionsDegree(n) = Val_Double(sa2(1))
                                TempCorrectionsMultiplier(n) = Val_Double(sa2(2))
                            End If
                            ' --------------------------------------------------------------------------- UNRECOGNIZED
                        Case Else
                            ScaleConfigLineUnrecognized(s1)
                            InitScaleDefaults()
                            Exit For
                    End Select
                End If
            Next
        Else
            MessageBox.Show("Configuration file not found" + vbCrLf + _
                            "Using default values.", _
                            "Message from Theremino-WaterQuality", _
                            MessageBoxButtons.OK)
            InitScaleDefaults()
            InitTemperatureDefaults()
        End If
        ' ----------------------------------------------------------------------------- test temperature correction data
        If TempSensorRaw.Length < 2 Or TempSensorDegree.Length < 2 Then
            MessageBox.Show("Temperature correction data are not valid" + vbCrLf + _
                            "Using default values.", _
                            "Message from Theremino-WaterQuality", _
                            MessageBoxButtons.OK)
            InitTemperatureDefaults()
        End If
        If TempCorrectionsDegree.Length < 2 Or TempCorrectionsMultiplier.Length < 2 Then
            MessageBox.Show("Temperature correction data are not valid" + vbCrLf + _
                            "Using default values.", _
                            "Message from Theremino-WaterQuality", _
                            MessageBoxButtons.OK)
            InitTemperatureDefaults()
        End If
        If TestRisingValues(TempSensorRaw) = False Or TestRisingValues(TempCorrectionsDegree) = False Then
            MessageBox.Show("Temperature correction data are not valid" + vbCrLf + _
                            "Using default values.", _
                            "Message from Theremino-WaterQuality", _
                            MessageBoxButtons.OK)
            InitTemperatureDefaults()
        End If
    End Sub

    Friend Function TestRisingValues(ByVal ar() As Double) As Boolean
        For i As Int32 = 0 To ar.Length - 2
            If ar(i) > ar(i + 1) Then Return False
        Next
        Return True
    End Function

    ' ==============================================================
    '  Useful functions
    ' ============================================================== 
    Friend Function ReplaceMultipleSpacesAndTabAndTrim(ByVal s As String) As String
        s = s.Replace(vbTab, " ")
        While s.Contains("  ")
            s = s.Replace("  ", " ")
        End While
        Return s.Trim
    End Function

    Friend Function RemoveComments(ByVal s As String) As String
        Dim i As Int32 = s.IndexOf("'")
        If i > 0 Then s = s.Remove(i).TrimEnd
        If i = 0 Then s = ""
        Return s
    End Function

    Private Sub ScaleConfigLineUnrecognized(ByVal s As String)
        MessageBox.Show("Unrecognized configuration line:  " + vbCrLf + vbCrLf + s + vbCrLf + vbCrLf + _
                        "The scale configuration is discarded and replaced with default values.", _
                        "Message from Theremino-WaterQuality", _
                        MessageBoxButtons.OK)
    End Sub

End Module
